<?php

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Import de skyblog
// Copyleft (C) 2006 BohwaZ, licence GNU/GPL, www.gnu.org

/* CHANGES

0.1.0 Version initiale
0.1.1 Correction de bug sur l'import du texte des billets
0.1.2 Correction de bug sur les dates
0.1.3 Correction de bug sur les images
0.2.0 Passage en class (plus propre) avec fonctions utilisateur -> rutilisable :)
0.3.0 Correction de bugs, export en bbcode/html
0.3.1 Message d'erreur si syntaxe html inconnue
0.4.0 Support nouvelle syntaxe html

*/

class importSkyblog
{
    // Tags  remplacer (les autres seront supprims)
    var $replaceTags = array(
        '<strong>' => '[g]', '</strong>' => '[/g]',
        '<em>' => "[i]", '</em>' => "[/i]",
        '<br />' => "\n",
        //'<span style="text-decoration: underline">([^<]+)</span>' => '[s]\\1[s]',
        '<a [^>]*href="([^"]+)"[^>]*>([^<]+)</a>' => '[a=\\1]\\2[/a]',

        // Images: ~IMAGE = chemin web de l'image, ~THUMB = chemin web de la miniature
        'imageLeft'=>'((~IMAGE||L))',
        'imageRight'=>'((~IMAGE||R))',
        'imageCenter'=>'((~IMAGE))',
        'thumbLeft'=>'[((~THUMB||L))|~IMAGE]',
        'thumbRight'=>'[((~THUMB||R))|~IMAGE]',
        'thumbCenter'=>'[((~THUMB))|~IMAGE]'
    );

    // Si il faut renvoyer les logs  une autre fonction, donner ici le nom de la fonction
    // Sinon laisser vide
    // La fonction prend en arguments $text et $level (warning/error/debug)
    var $logFunction = '';

    // Fonction utilisateur pour l'import des fichiers images en local
    // Laisser vide pour laisser les images sur skyblog
    // La fonction prend en arguments $image et ventuellement $thumb
    // Elle doit renvoyer le chemin web vers le rpertoire d'images local (ex: 'images/skyblog/')
    var $imagesFunction = '';

    // Variables  usage interne,  ne modifier qu'en cas de changement du moteur de skyblog
    var $validBlogPattern = '^[a-z0-9_-]+$';
    var $blogUrl = 'http://BLOG.skyblog.com/';
    var $months = array("janvier"=>1,"fvrier"=>2,"mars"=>3,"avril"=>4,"mai"=>5,"juin"=>6,"juillet"=>7,
        "aot"=>8,"septembre"=>9,"octobre"=>10,"novembre"=>11,"dcembre"=>12);

    var $log = array();
    var $general = array();

    // Fonction de log
    function Log($text,$level="")
    {
        if($level == 'err') $level = 'error';
        elseif($level == 'warn') $level = 'warning';
        else $level = 'debug';

        if(!empty($this->logFunction)) {
            call_user_func($this->logFunction,$text,$level);
            if($level == 'error') return false;
            return true;
        }

        $this->log[] = array($text,$level);
        if($level == 'error') {
            echo '<h3 style="color: red">Erreur</h3><p>'.$text.'</p>';
            return false;
        }
        elseif($level == 'warning')
            echo '<strong style="color: red">Warning:</strong> '.$text.'<br />';
        else
            echo $text.'<br />';

        return true;
    }

    // On remplace les tags html par les tags maison
    function replaceTags($text)
    {
        $text = stripslashes($text);
        $text = ereg_replace('onclick="([^"]*)"', '', $text);

        if($this->outputFormat == "html")
        {
            $text = ereg_replace('<div style="margin:0;padding:0;text-align:(center|left|right);">([^<]+)</div>',
                '<p style="text-align: \\1;">\\2</p>', $text);
            $text = ereg_replace('<div([^>]*)>', '', $text);
            $text = ereg_replace('</div>', '', $text);
        }
        else
        {
            $text = strip_tags($text, '<br><a><strong><em>');
            foreach($this->replaceTags as $tag=>$replace)
            {
                if(strpos($tag,'<') !== false)
                {
                    $text = ereg_replace($tag,$replace,$text);
                }
            }
            $text = strip_tags($text);

            if(function_exists('html_entity_decode'))
                $text = html_entity_decode($text);
        }

        return $text;
    }

    // Renvoie l'url d'un skyblog  partir de son pseudo
    function getBlogUrl($blog)
    {
        $url = str_replace('BLOG',$blog,$this->blogUrl);
        return $url;
    }

    // Rcupre la premire page: on ne parse qu'une fois les infos javascript etc
    function getHome($blogId)
    {
        if(!ereg($this->validBlogPattern, $blogId))
            return $this->Log("Adresse de skyblog invalide.",'err');

        $url = $this->getBlogUrl($blogId);
        $this->Log('Connexion  '.$url);

        // Si la connexion choue, surement que l'ouverture d'URL extrieure est dsactive dans PHP
        $content = @file($url.'?import-skyblog');
        if(!$content || !is_array($content) || @count($content) < 10) return $this->Log('Impossible de joindre '.$url,'err');
        $this->Log('Connexion tablie.');

        $general = array();
        $general['blogId'] = $blogId;
        $general['blogroll'] = array();

        foreach($content as $line)
        {
            $line = trim($line);

            // Ici on ne traite pas les articles
            if(ereg('^<div id="content">$',$line)) break;

            // Si une donne est sur plusieurs lignes
            if($open)
            {
                // Rcupration de la blogroll (skyblogs prfrs et amis)
                if($open == "blogroll")
                {
                    while(ereg('<a href="http://([^.]+).skyblog.com/" ', $line, $out))
                    {
                        $item = $out[1];
                        // On ne l'ajoute  la blogroll que s'il ne l'est pas dj (vite les doublons entre amis et favoris)
                        if(!in_array($item, $general[$open]))
                        {
                            $this->Log('[blogroll] '.$this->getBlogUrl($item));
                            $general[$open][] = $item;
                        }
                        $line = str_replace($out[0], '', $line);
                    }
                }
                else
                {
                    $this->Log('[general] '.$open.': '.strip_tags($line));
                    $general[$open].= strip_tags($line);
                }

                if(ereg('</dd>', $line)) unset($open);
                continue;
            }

            // Vrifications d'usage
            if(ereg('/x-css/404.css',$line)) return $this->Log("Ce skyblog n'existe pas. Vrifiez l'adresse.",'err');
            if(ereg('/x-css/desactivation.css',$line)) return $this->Log("Ce skyblog est dsactiv. Vrifiez l'adresse.",'err');

            // Titre du skyblog, on pourrait le rcuprer plus tard, mais l il est sur une ligne et sans HTML
            if(ereg("<title>Skyblog de $blogId : ([^<]+)</title>",$line,$out)) {
                $this->Log('[general] Titre: '.$out[1]);
                $general['title'] = $out[1];
            }

            // L'import de skyblog music n'est pas encore possible
            if(ereg('^<title>Skyblog Music de ', $line)) {
                return $this->Log("Vous ne pouvez pas importer de skyblog music!", 'err');
            }

            // Donnes sur plusieurs lignes
            if(ereg('^<dt> Description&nbsp;: </dt>$',$line)) { $open = "description"; continue;}
            if(ereg('^<dt> Date de cration&nbsp;: </dt>$',$line)) { $open = "creation"; continue; }
            if(ereg('^<dt> Dernire mise  jour&nbsp;: </dt>$',$line)) { $open = "updated"; continue; }
            if(ereg('^<dl id="amis">$',$line)) { $open = "blogroll"; continue; }
            if(ereg('^<dl id="favoris">$',$line)) { $open = "blogroll"; continue; }

            // Donnes javascript
            if(!ereg('^var ',$line)) continue;

            // Version du template selon skyblog
            // Avec la V2 apparaissent les amis, les skyblog music, et de nouveaux templates
            // Si skyblog garde une numrotation fiable on pourra dans le futur parser plusieurs versions avec la mme class
            if(ereg('^var tpl_ver = 2;$', $line)) {
                $parseVersion = 2;
            }

            // On vrifie qu'il n'y a pas d'incohrences
            if(ereg('^var blog_spath = "http://([^.]+)\.skyblog\.com/";',$line,$out)) {
                if($out[1] != $blogId) return $this->Log('Problme inconnu: ce skyblog ne correspond pas  celui demand.','err');
                $general['blogUrl'] = $url;
            }
            if(ereg('^var blog_cur_page = ([0-9]+);$', $line, $out)) {
                if(intval($out[1]) != 1) return $this->Log("Problme inconnu: la page d'accueil n'est pas la premire page.",'err');
            }
            if(ereg('^var use_smileys = ([0-9]+);$', $line, $out)) {
                if(intval($out[1]) != 0) $this->Log("Etrange: Les smileys graphiques sont activs (exprimental, normalement dsactiv)",'warn');
            }
            if(ereg('^var blog_no_pub = ([0-9]+);$', $line, $out)) {
                if(intval($out[1]) != 0) return $this->Log("Problme inconnu: ce skyblog n'a pas de publicit.",'err');
            }
            if(ereg('^var skyblog_officiel = ([0-9]+);', $line, $out)) {
                if(intval($out[1]) != 0) return $this->Log("Vous ne pouvez pas importer un skyblog officiel.",'err');
            }
            if(ereg('^var skyblog_ope = "([^"]*)";$', $line, $out)) {
                if(!empty($out[1])) return $this->Log("Vous ne pouvez pas importer un skyblog correspondant  une opration pub.", 'err');
            }

            // Rcupration infos utiles pour la suite
            if(ereg('^var nb_articles = ([0-9]+);$',$line,$out)) {
                $general['nb_articles'] = $out[1];
                $this->Log('[general] Nombre d\'articles: '.$out[1]);
            }
            if(ereg('^var blog_nb_page = ([0-9]+);$',$line,$out)) {
                $general['total_pages'] = $out[1];
                $this->Log('[general] Nombre de pages: '.$out[1]);
            }
            if(ereg('^var idtpl = ([0-9]+);$',$line,$out)) {
                $general['template_id'] = $out[1];
                $this->Log('[general] Template utilis: #'.$out[1]);
            }
            if(ereg('^var blog_with_comments = ([0-9]+);$',$line,$out)) {
                if(intval($out[1]) == 1) $general['comments'] = true;
                else $general['comments'] = false;
                $this->Log('[general] Commentaires: '.($out[1] ? 'Activs' : 'Dsactivs'));
            }
        }

        if(empty($parseVersion) || $parseVersion != 2)
            return $this->Log("Ce skyblog utilise des templates inconnus. Srement une ancienne version. ".
                "Essayez de regnerer le skyblog (cliquer sur le bouton publier dans son admin) et ".
                "de re-tenter l'import quelques jours plus tard.", 'err');

        // Rcupration du nombre de commentaires par article
        if($general['comments'])
        {
            $file = @file($url.'includenbcomment.js?import-skyblog');
            if(!$file) $this->Log('Impossible de rcuprer le javascript donnant le nombre de commentaires.','warn');
            $file = implode("",$file);
            $file = explode(";",$file);
            $nb_comments = array();
            $general['nb_comments'] = 0;
            foreach($file as $comment)
            {
                if(ereg("^n\[([0-9]+)\]=([0-9]+)$",trim($comment),$out)) {
                    $general['nb_comments'] += intval($out[2]);
                    $nb_comments[$out[1]] = $out[2];
                }
            }
        }

        $general['comments_list'] = $nb_comments;
        $this->general = $general;

        $articles = $this->getArticles($content);
        return $articles;
    }

    function getArticles($content)
    {
        $general = $this->general;

        foreach($content as $line)
        {
            $line = trim($line);

            if($open)
            {
                if(ereg('^<h2>', $line))
                {
                    $articles[$id]['title'] = strip_tags($line);

                    if(function_exists('html_entity_decode'))
                        $articles[$id]['title'] = html_entity_decode($articles[$id]['title']);

                    if(empty($articles[$id]['title'])) {
                        $articles[$id]['title'] = "Article #$id";
                        $this->Log("L'article #".$id." n'a pas de titre.", 'warn');
                    }
                    $this->Log('[article] Import de #'.$id.' : "'.$articles[$id]['title'].'"');
                }
                elseif(ereg('^(<div>|<div class="image-container)',$line) && empty($aContent))
                {
                    $aContent = $line;
                }
                elseif($general['comments'] && ereg('^<div class="commentaires" id="c-([0-9]+)">',$line))
                {
                    // Import des commentaires de l'article
                    $articles[$id]['comments'] = $this->importComments($id, 1);
                }
                elseif(ereg('^<div class="date"><div> Post le ([^<]+)</div>',$line,$out))
                {
                    // Import de la date
                    $articles[$id]['date'] = $this->importDate($out[1]);

                    // Import des images avec miniature
                    while(ereg('<div class="image-container (center|left|right)">'.
                        '<a href="photo.html\?pics/([^.]+\.jpg)[^>]+><img src="pics/([^.]+\.jpg)"[^>]+></a></div>', $aContent, $out))
                    {
                        $path = $this->importImages($out[2], $out[3]);

                        if($this->outputFormat == 'html')
                        {
                            if($out[1] == "left") $tag = '<p class="img" style="float: left; padding-right: 10px;"><a href="~IMAGE"><img src="~THUMB" alt="" /></a></p>';
                            elseif($out[1] == "right") $tag = '<p class="img" style="float: right; padding-left: 10px;"><a href="~IMAGE"><img src="~THUMB" alt="" /></a></p>';
                            else $tag = '<p class="img" style="text-align: center; margin: 8px;"><a href="~IMAGE"><img src="~THUMB" alt="" /></a></p>';
                        }
                        else
                        {
                            if($out[1] == "left") $tag = $this->replaceTags['thumbLeft'];
                            elseif($out[1] == "right") $tag = $this->replaceTags['thumbRight'];
                            else $tag = $this->replaceTags['thumbCenter'];
                        }

                        $tag = "\n\n".strtr($tag,array('~THUMB'=>$path.$out[3],'~IMAGE'=>$path.$out[2]))."\n\n";
                        $aContent = str_replace($out[0],$tag,$aContent);
                    }
                    // Import des images sans miniature
                    while(ereg('<div class="image-container (center|left|right)">'.
                        '<img src="pics/([^.]+\.(jpg|gif))"[^>]+></div>',$aContent,$out))
                    {
                        $path = $this->importImages($out[2]);

                        if($this->outputFormat == 'html')
                        {
                            if($out[1] == "left") $tag = '<p class="img" style="float: left; padding-right: 10px;"><img src="~IMAGE" alt="" /></p>';
                            elseif($out[1] == "right") $tag = '<p class="img" style="float: right; padding-left: 10px;"><img src="~IMAGE" alt="" /</p>';
                            else $tag = '<p class="img" style="text-align: center; margin: 8px;"><img src="~IMAGE" alt="" /></p>';
                        }
                        else
                        {
                            if($out[1] == "left") $tag = $this->replaceTags['imageLeft'];
                            elseif($out[1] == "right") $tag = $this->replaceTags['imageRight'];
                            else $tag = $this->replaceTags['imageCenter'];
                        }

                        $tag = "\n\n".str_replace('~IMAGE',$path.$out[2],$tag)."\n\n";
                        $aContent = str_replace($out[0],$tag,$aContent);
                    }

                    // Conversion des tags...
                    $aContent = $this->replaceTags($aContent);

                    if(function_exists('html_entity_decode'))
                        $aContent = html_entity_decode($aContent);

                    $articles[$id]['content'] = trim($aContent);
                    unset($open);
                    unset($id);
                    unset($aContent);
                }
                elseif(!empty($aContent)) $aContent.= $line;

                continue;
            }

            if(ereg('^var blog_spath="http://([^.]+)\.skyblog\.com/";',$line,$out)) {
                if(intval($out[1]) != $general['blogId'])
                    return $this->Log("Problme inconnu: ce skyblog ne correspond pas  celui demand.",'err');
            }
            if(ereg('^var nb_articles=([0-9]+);$',$line,$out)) {
                if(intval($out[1]) != $general['nb_articles'])
                    return $this->Log("Des articles ont t ajout aux skyblog depuis le dbut de son import. ".
                        "Impossible de continuer dans ces conditions!",'err');
            }

            if(ereg('^<div class="article" id="a-([0-9]+)">$', $line, $out)) { $open = "article"; $id = $out[1]; continue; }
        }

        return $articles;
    }

    function importImages($image,$thumb="")
    {
        if(!isset($this->general['nb_images']))
            $this->general['nb_images'] = 0;

        if(!empty($this->imagesFunction))
        {
            $path = call_user_func($this->imagesFunction,$image,$thumb);
            if($path !== false) {
                $this->Log('[image] Import russi de '.$image.' ('.$this->general['nb_images'].')');
                $this->general['nb_images']++;
                return $path;
            }
            else $this->Log('[image] Impossible d\'importer: '.$image);
        }

        $path = $this->general['blogUrl'].'pics/';
        return $path;
    }

    function importComments($id,$page,$count=1)
    {
        $comments = array();
        $blogId = $this->general['blogId'];
        #$tpl = $this->general['template'];

        $file = file($this->general['blogUrl'].$id.'_comment_'.$page.'.html?import-skyblog');

        foreach($file as $line)
        {
            $line = trim($line);

            if($open)
            {
                if(ereg('^<hr />$',$line))
                {
                    $this->Log('[commentaire] Import de #'.$idc.' (sur article #'.$id.')');
                    unset($open);
                    unset($idc);
                    $count += 1;
                    continue;
                }
                $line = str_replace('<br />', "\n", $line);
                $line = strip_tags($line);

                if(function_exists('html_entity_decode'))
                    $line = html_entity_decode($line);

                $comments[$idc]['content'].= $line;
            }

            // Protection contre comportement trange de la plate-forme skyblog...
            if(ereg('^var id_article=([0-9]+);$',$line,$out)) {
                if(intval($out[1]) != $id) return array();
            }
            if(ereg('^var blog_spath="http://([^.]+).skyblog.com/";$',$line,$out)) {
                if($out[1] != $blogId) return array();
            }
            if(ereg('^var cur_page=([0-9]+);$',$line,$out)) {
                if(intval($out[1]) != $page) return array();
            }
            if(ereg('^var nb_page=([0-9]+);$', $line, $out)) {
                $total = intval($out[1]);
                continue;
            }

            if(ereg('<h3>De <strong><a href="mailto:([^"]+)">([^<]+)</a></strong>, post le <strong>([^<]+)</strong></h3>$', $line, $out)
            || ereg('<p class="title">De <strong><a href="mailto:([^"]+)">([^<]+)</a></strong>, post le <strong>([^<]+)</strong></p>$', $line, $out))
            {
                $open = true;
                $idc = $count;
                $comments[$idc]['name'] = $out[2];
                $comments[$idc]['email'] = str_replace('%40', '@', $out[1]);
                $comments[$idc]['date'] = $this->importDate($out[3]);
                continue;
            }
            if(ereg('^<h3>De <strong>([^<]+)</strong>, post le <strong>([^<]+)</strong></h3>$', $line, $out)
            || ereg('^<p class="title">De <strong>([^<]+)</strong>, post le <strong>([^<]+)</strong></p>$', $line, $out))
            {
                $open = true;
                $idc = $count;
                $comments[$idc]['name'] = $out[1];
                $comments[$idc]['date'] = $this->importDate($out[2]);
                continue;
            }
        }

        if($page < $total)
        {
            $next = $this->importComments($id, $page+1, $count);
            $comments = array_merge($comments, $next);
        }

        return $comments;
    }

    function importDate($date)
    {
        $mois = $this->months;
        $date = explode(" ",trim($date));
        $heure = explode(":",$date[5]);
        $time = mktime($heure[0],$heure[1],0,$mois[strtolower($date[2])],$date[1],$date[3]);
        return $time;
    }

    function getPage($p)
    {
        $p = intval($p);
        $content = @file($this->general['blogUrl'].$p.'.html?import-skyblog');
        if(!$content || !is_array($content) || @count($content) < 10)
            return $this->Log('Impossible de rcuprer '.$this->general['blogUrl'].$p.'.html','err');

        return $this->getArticles($content);
    }
}

?>
